@@ -6,8 +6,8 @@ I/O device.
 __author__ = "Dennis van Gils"
 __authoremail__ = "vangils.dennis@gmail.com"
 __url__ = "https://github.com/Dennis-van-Gils/python-dvg-qdeviceio"
-__date__ = "15-07-2020"
-__version__ = "0.2.1"  # v0.0.1 on PyPI is based on prototype DvG_dev_Base__pyqt_lib.py v1.3.3
+__date__ = "17-07-2020"
+__version__ = "0.2.2"
 # pylint: disable=protected-access
 
 from enum import IntEnum, unique
@@ -26,6 +26,15 @@ import sys
 import threading
 from functools import wraps
 
+import numpy as np
+from PyQt5 import QtCore
+from dvg_debug_functions import (
+    print_fancy_traceback as pft,
+    dprint,
+    tprint,
+    ANSI,
+)
+
 running_coverage = "coverage" in sys.modules
 if running_coverage:
     print("\nCode coverage test detected\n")
@@ -41,15 +50,6 @@ def _coverage_resolve_trace(fn):
     return wrapped
 
 
-import numpy as np
-from PyQt5 import QtCore
-from dvg_debug_functions import (
-    print_fancy_traceback as pft,
-    dprint,
-    tprint,
-    ANSI,
-)
-
 # Short-hand alias for DEBUG information
 def _cur_thread_name():
     return QtCore.QThread.currentThread().objectName()
@@ -432,7 +432,7 @@ class QDeviceIO(QtCore.QObject):
 
         elif not self.dev.is_alive:
             dprint(
-                "\nWorker_DAQ  %s: WARNING - Device is not alive.\n"
+                "Worker_DAQ  %s: WARNING - Device is not alive.\n"
                 % self.dev.name,
                 ANSI.RED,
             )
@@ -498,7 +498,7 @@ class QDeviceIO(QtCore.QObject):
 
         elif not self.dev.is_alive:
             dprint(
-                "\nWorker_jobs %s: WARNING - Device is not alive.\n"
+                "Worker_jobs %s: WARNING - Device is not alive.\n"
                 % self.dev.name,
                 ANSI.RED,
             )
@@ -579,35 +579,40 @@ class QDeviceIO(QtCore.QObject):
             )
             return True
 
-        if self.worker_DAQ.debug:
-            tprint(
-                "Worker_DAQ  %s: stop requested..." % self.dev.name, ANSI.WHITE,
-            )
+        if not self.worker_DAQ._has_finished:
+            if self.worker_DAQ.debug:
+                tprint(
+                    "Worker_DAQ  %s: stop requested..." % self.dev.name,
+                    ANSI.WHITE,
+                )
 
-        if self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
-            # The QTimer inside the INTERNAL_TIMER '_do_work()'-routine /must/
-            # be stopped from within the worker_DAQ thread. Hence, we must use a
-            # signal from out of this different thread.
-            self._signal_stop_worker_DAQ.emit()
+            if self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
+                # The QTimer inside the INTERNAL_TIMER '_do_work()'-routine
+                # /must/ be stopped from within the worker_DAQ thread. Hence,
+                # we must use a signal from out of this different thread.
+                self._signal_stop_worker_DAQ.emit()
 
-        elif self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
-            # The QWaitCondition inside the SINGLE_SHOT_WAKE_UP '_do_work()'-
-            # routine will likely have locked worker_DAQ. Hence, a
-            # '_signal_stop_worker_DAQ' signal as above might not get handled by
-            # worker_DAQ when emitted from out of this thread. Instead,
-            # we directly call '_stop()' from out of this different thread,
-            # which is perfectly fine for SINGLE_SHOT_WAKE_UP as per my design.
-            self.worker_DAQ._stop()
+            elif (
+                self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP
+            ):
+                # The QWaitCondition inside the SINGLE_SHOT_WAKE_UP '_do_work()'
+                # routine will likely have locked worker_DAQ. Hence, a
+                # '_signal_stop_worker_DAQ' signal as above might not get
+                # handled by worker_DAQ when emitted from out of this thread.
+                # Instead, we directly call '_stop()' from out of this different
+                # thread, which is perfectly fine for SINGLE_SHOT_WAKE_UP as per
+                # my design.
+                self.worker_DAQ._stop()
 
-        elif self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
-            # We directly call '_stop()' from out of this different thread,
-            # which is perfectly fine for CONTINUOUS as per my design.
-            self.worker_DAQ._stop()
+            elif self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
+                # We directly call '_stop()' from out of this different thread,
+                # which is perfectly fine for CONTINUOUS as per my design.
+                self.worker_DAQ._stop()
 
-        # Wait for worker_DAQ to confirm having stopped
-        locker_wait = QtCore.QMutexLocker(self._mutex_wait_worker_DAQ)
-        self._qwc_worker_DAQ_stopped.wait(self._mutex_wait_worker_DAQ)
-        locker_wait.unlock()
+            # Wait for worker_DAQ to confirm having stopped
+            locker_wait = QtCore.QMutexLocker(self._mutex_wait_worker_DAQ)
+            self._qwc_worker_DAQ_stopped.wait(self._mutex_wait_worker_DAQ)
+            locker_wait.unlock()
 
         self._thread_DAQ.quit()
         print(
@@ -655,23 +660,25 @@ class QDeviceIO(QtCore.QObject):
             )
             return True
 
-        if self.worker_jobs.debug:
-            tprint(
-                "Worker_jobs %s: stop requested..." % self.dev.name, ANSI.WHITE,
-            )
+        if not self.worker_jobs._has_finished:
+            if self.worker_jobs.debug:
+                tprint(
+                    "Worker_jobs %s: stop requested..." % self.dev.name,
+                    ANSI.WHITE,
+                )
 
-        # The QWaitCondition inside the SINGLE_SHOT_WAKE_UP '_do_work()'-
-        # routine will likely have locked worker_DAQ. Hence, a
-        # '_signal_stop_worker_DAQ' signal might not get handled by
-        # worker_DAQ when emitted from out of this thread. Instead,
-        # we directly call '_stop()' from out of this different thread,
-        # which is perfectly fine as per my design.
-        self.worker_jobs._stop()
+            # The QWaitCondition inside the SINGLE_SHOT_WAKE_UP '_do_work()'-
+            # routine will likely have locked worker_DAQ. Hence, a
+            # '_signal_stop_worker_DAQ' signal might not get handled by
+            # worker_DAQ when emitted from out of this thread. Instead,
+            # we directly call '_stop()' from out of this different thread,
+            # which is perfectly fine as per my design.
+            self.worker_jobs._stop()
 
-        # Wait for worker_jobs to confirm having stopped
-        locker_wait = QtCore.QMutexLocker(self._mutex_wait_worker_jobs)
-        self._qwc_worker_jobs_stopped.wait(self._mutex_wait_worker_jobs)
-        locker_wait.unlock()
+            # Wait for worker_jobs to confirm having stopped
+            locker_wait = QtCore.QMutexLocker(self._mutex_wait_worker_jobs)
+            self._qwc_worker_jobs_stopped.wait(self._mutex_wait_worker_jobs)
+            locker_wait.unlock()
 
         self._thread_jobs.quit()
         print(
@@ -977,6 +984,7 @@ class Worker_DAQ(QtCore.QObject):
         self.calc_DAQ_rate_every_N_iter = calc_DAQ_rate_every_N_iter
 
         self._started_okay = None
+        self._has_finished = False
 
         # Members specifically for INTERNAL_TIMER
         if self._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
@@ -1096,6 +1104,8 @@ class Worker_DAQ(QtCore.QObject):
                 lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
+            self._has_finished = True
+
         # CONTINUOUS
         elif self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             while self._running:
@@ -1154,6 +1164,8 @@ class Worker_DAQ(QtCore.QObject):
                 lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
+            self._has_finished = True
+
     @_coverage_resolve_trace
     @QtCore.pyqtSlot()
     def _perform_DAQ(self):
@@ -1193,7 +1205,7 @@ class Worker_DAQ(QtCore.QObject):
         # Check the not alive counter
         if self.qdev.not_alive_counter_DAQ >= self.critical_not_alive_count:
             dprint(
-                "\nWorker_DAQ %s: Lost connection to device.\n" % self.dev.name,
+                "Worker_DAQ  %s: Lost connection to device." % self.dev.name,
                 ANSI.RED,
             )
             self.dev.is_alive = False
@@ -1208,14 +1220,22 @@ class Worker_DAQ(QtCore.QObject):
         #   User-supplied DAQ function
         # ----------------------------------
 
-        if not self.DAQ_function is None:
-            if self.DAQ_function():
-                # Did return True, hence was successfull
-                # --> Reset the 'not alive' counter
-                self.qdev.not_alive_counter_DAQ = 0
+        if self.DAQ_function is not None:
+            try:
+                success = self.DAQ_function()
+            except Exception as err:  # pylint: disable=broad-except
+                pft(err)
+                dprint(
+                    "@ Worker_DAQ %s\n" % self.dev.name, ANSI.RED,
+                )
             else:
-                # Did return False, hence was unsuccessfull
-                self.qdev.not_alive_counter_DAQ += 1
+                if success:
+                    # Did return True, hence was successfull
+                    # --> Reset the 'not alive' counter
+                    self.qdev.not_alive_counter_DAQ = 0
+                else:
+                    # Did return False, hence was unsuccessfull
+                    self.qdev.not_alive_counter_DAQ += 1
 
         # ----------------------------------
         #   End user-supplied DAQ function
@@ -1255,6 +1275,8 @@ class Worker_DAQ(QtCore.QObject):
                 lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
+            self._has_finished = True
+
         elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             self._running = False
             self._qwc.wakeAll()  # Wake up for the final time
@@ -1455,6 +1477,7 @@ class Worker_jobs(QtCore.QObject):
 
         self.jobs_function = jobs_function
         self._started_okay = None
+        self._has_finished = False
 
         self._running = True
         self._qwc = QtCore.QWaitCondition()
@@ -1545,6 +1568,8 @@ class Worker_jobs(QtCore.QObject):
             lambda: self.qdev._qwc_worker_jobs_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
         )
 
+        self._has_finished = True
+
     @_coverage_resolve_trace
     @QtCore.pyqtSlot()
     def _perform_jobs(self):
@@ -1591,6 +1616,9 @@ class Worker_jobs(QtCore.QObject):
                         func(*args)
                     except Exception as err:  # pylint: disable=broad-except
                         pft(err)
+                        dprint(
+                            "@ Worker_jobs %s\n" % self.dev.name, ANSI.RED,
+                        )
                 else:
                     # User-supplied job processing
                     self.jobs_function(func, args)
